/*
 * Copyright (c) 2002, 2003, 2004 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http://www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http://www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License and the Policies for
 * specific language governing the use of this program.
 */
package org.activebpel.samples.loan_approval.util;
import java.io.File;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import org.jaxen.XPath;
import org.jaxen.dom.DOMXPath;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Holds configuration information read from an XML file, returns values
 * when given XPath expressions, and saves values when told.
 * <p>
 * Every time a new RuntimeParams object is created, the config file is
 * re-opened and re-parsed. This is a Good Thing(tm) because we want changes
 * to the config file to take place immediately.
 *
 * @author Jim Menard, <a href="mailto:jimm@activebpel.org">jimm@activebpel.org</a>
 */
public class RuntimeParams {

/** Default directory where the config file is stored. */
private static final String DEFAULT_TEMPDIR =
    System.getProperty("java.io.tmpdir");
/** Default config file name. */
private static final String DEFAULT_CONFIG_FILE = "bpel_example_config.xml";

protected Document doc;
protected File configFile;

/**
 * Constructor. Reads in the XML config file
 * <code>DEFAULT_CONFIG_FILE</code> inside <code>DEFAULT_TEMPDIR</code>
 * and holds on to it as a <code>Document</code>.
 */
public RuntimeParams() throws Exception {
    this(DEFAULT_CONFIG_FILE, DEFAULT_TEMPDIR);
}

/**
 * Constructor. Reads in the XML config file <var>configFileName</var>
 * inside <code>DEFAULT_TEMPDIR</code> and holds on to it as a
 * <code>Document</code>.
 *
 * @param configFileName the XML file's name
 */
public RuntimeParams(String configFileName) throws Exception {
    this(configFileName, DEFAULT_TEMPDIR);
}

/**
 * Constructor. Reads in the XML config file <var>configFileName</var>
 * inside <var>dirPath</var> and holds on to it as a <code>Document</code>.
 *
 * @param configFileName the XML file's name
 * @param dirPath path to the directory in which <var>configFileName</var>
 * resides
 */
public RuntimeParams(String configFileName, String dirPath)
    throws Exception
{
    DocumentBuilder builder =
	DocumentBuilderFactory.newInstance().newDocumentBuilder();
    configFile = new File(dirPath, configFileName);
    doc = builder.parse(configFile);
}

/**
 * Given an <var>xpath</var>, returns the value of that element.
 *
 * @param xpath an XPath expression
 * @return the value
 */
public String get(String xpath) throws Exception {
    XPath xp = new DOMXPath(xpath);
    Node node = (Node)xp.selectSingleNode(doc);
    return node.getNodeValue().trim();
}

/**
 * Given a lookup <var>xpath</var>, returns the value of the text inside
 * the specified element.
 *
 * @param xpath an XPath expression
 * @return the value of the text inside the specified element
 */
public String getText(String xpath) throws Exception {
    return get(xpath + "/text()[1]");
}

/**
 * Given a lookup <var>xpath</var> and an attribute name, returns the value
 * of that attribute.
 *
 * @param xpath an XPath expression
 * @param attrName an attribute name
 * @return the value of the text inside the specified element
 */
public String getAttr(String xpath, String attrName) throws Exception {
    return get(xpath + "/@" + attrName);
}

/**
 * Sets the <var>value</var> of <var>xpath</var>.
 *
 * @param xpath an XPath expression
 * @param value any string
 */
public void set(String xpath, String value) throws Exception {
    XPath xp = new DOMXPath(xpath);
    Node node = (Node)xp.selectSingleNode(doc);
    node.setNodeValue(value);
}

/**
 * Sets the <var>value</var> of the text inside <var>xpath</var>.
 *
 * @param xpath an XPath expression
 * @param value any string
 */
public void setText(String xpath, String value) throws Exception {
    set(xpath + "/text()[1]", value);
}

/**
 * Sets the attribute <var>attrName</var> of <var>xpath</var> to
 * <var>value</var>.
 *
 * @param xpath an XPath expression
 * @param attrName an attribute name
 * @param value any string
 */
public void setAttr(String xpath, String attrName, String value)
    throws Exception
{
    set(xpath + "/@" + attrName, value);
}

/**
 * Saves document values back to the config XML file.
 */
public void save() throws Exception {
    Source source = new DOMSource(doc);
    Result result = new StreamResult(configFile);
    Transformer xformer = TransformerFactory.newInstance().newTransformer();
    xformer.transform(source, result);
}
}
